@@ -5,7 +5,7 @@
 See ``dir(unpythonic)`` and submodule docstrings for more.
 """
 
-__version__ = '0.8.3'
+__version__ = '0.8.5'
 
 from . import rc
 
@@ -16,6 +16,8 @@ from .dynscope import *
 from .ec import *
 from .fun import *
 from .fup import *
+from .gmemo import *
+from .gtco import *
 from .it import *
 from .let import *  # no guarantees on evaluation order (before Python 3.6), nice syntax
 
