@@ -15,10 +15,10 @@ and (stream-scan) in SRFI-41.
 
 __all__ = ["scanl", "scanr", "scanl1", "scanr1",
            "foldl", "foldr", "reducel", "reducer",
-           "mapr", "zipr", "map_longest", "mapr_longest", "zipr_longest",
+           "map_longest", "mapr", "zipr", "mapr_longest", "zipr_longest",
            "flatmap", "uniqify", "uniq",
            "take", "drop", "split_at", "unpack",
-           "tail", "first", "second", "nth", "last",
+           "tail", "first", "second", "nth", "last", "scons",
            "flatten", "flatten1", "flatten_in",
            "iterate", "iterate1"]
 
@@ -27,7 +27,9 @@ from functools import partial
 from itertools import tee, islice, zip_longest, starmap, chain, filterfalse, groupby
 from collections import deque
 
-# require at least one iterable to make this work seamlessly with curry.
+# Require at least one iterable to make this work seamlessly with curry.
+# We take this approach with any new function families Python doesn't provide
+# (mainly scans and folds).
 def scanl(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
     """Scan (accumulate), optionally with multiple input iterables.
 
@@ -43,7 +45,7 @@ def scanl(proc, init, iterable0, *iterables, longest=False, fillvalue=None):
     By default, terminate when the shortest input runs out. To terminate on
     longest input, use ``longest=True`` and optionally provide a ``fillvalue``.
 
-    Returns a generator, which (in pseudocode)::
+    Returns a generator, which (roughly)::
 
         acc = init
         yield acc
@@ -143,14 +145,8 @@ def reducer(proc, sequence, init=None):
     """
     return reducel(proc, reversed(sequence), init)
 
-def mapr(func, *sequences):
-    """Like map, but walk each sequence from the right."""
-    return map(func, *(reversed(s) for s in sequences))
-
-def zipr(*sequences):
-    """Like zip, but walk each sequence from the right."""
-    return zip(*(reversed(s) for s in sequences))
-
+# Here we complete an existing set of functions (map, zip, zip_longest),
+# so consistency wins over curry-friendliness.
 def map_longest(func, *iterables, fillvalue=None):
     """Like map, but terminate on the longest input.
 
@@ -159,6 +155,14 @@ def map_longest(func, *iterables, fillvalue=None):
     """
     return starmap(func, zip_longest(*iterables, fillvalue=fillvalue))
 
+def mapr(func, *sequences):
+    """Like map, but walk each sequence from the right."""
+    return map(func, *(reversed(s) for s in sequences))
+
+def zipr(*sequences):
+    """Like zip, but walk each sequence from the right."""
+    return zip(*(reversed(s) for s in sequences))
+
 def mapr_longest(func, *sequences, fillvalue=None):
     """Like map_longest, but walk each sequence from the right."""
     return map_longest(func, *(reversed(s) for s in sequences), fillvalue=fillvalue)
@@ -198,11 +202,11 @@ def flatmap(f, iterable0, *iterables):
         assert tuple(flatmap(sum_and_diff, (10, 20, 30), (1, 2, 3))) == \\
                (11, 9, 22, 18, 33, 27)
     """
-    yield from chain.from_iterable(map(f, iterable0, *iterables))
+    return chain.from_iterable(map(f, iterable0, *iterables))
 #    for xs in map(f, iterable0, *iterables):
 #        yield from xs
 
-def uniqify(iterable, key=None):
+def uniqify(iterable, *, key=None):
     """Skip duplicates in iterable.
 
     Returns a generator that yields unique items from iterable, preserving
@@ -227,7 +231,7 @@ def uniqify(iterable, key=None):
                 seen_add(k)
                 yield e
 
-def uniq(iterable, key=None):
+def uniq(iterable, *, key=None):
     """Like uniqify, but for consecutive duplicates only.
 
     Named after the *nix utility.
@@ -244,6 +248,8 @@ def take(n, iterable):
 
     This is ``take`` from ``itertools`` recipes.
     """
+    if not isinstance(n, int):
+        raise TypeError("expected integer n, got {} with value {}".format(type(n), n))
     if n < 0:
         raise ValueError("expected n >= 0, got {}".format(n))
     return islice(iter(iterable), n)
@@ -255,6 +261,8 @@ def drop(n, iterable):
 
     This is ``consume`` from ``itertools`` recipes.
     """
+    if not isinstance(n, int):
+        raise TypeError("expected integer n, got {} with value {}".format(type(n), n))
     if n < 0:
         raise ValueError("expected n >= 0, got {}".format(n))
     it = iter(iterable)
@@ -281,6 +289,8 @@ def split_at(n, iterable):
         assert a == tuple(range(3))
         assert b == ()
     """
+    if not isinstance(n, int):
+        raise TypeError("expected integer n, got {} with value {}".format(type(n), n))
     if n < 0:
         raise ValueError("expected n >= 0, got {}".format(n))
     ia, ib = tee(iter(iterable))
@@ -316,9 +326,13 @@ def unpack(n, iterable, *, k=None, fillvalue=None):
     If ``k > n`` (skip some items after the first n), then after extraction,
     the tail is formed by fast-forwarding the iterator using ``drop``.
     """
+    if not isinstance(n, int):
+        raise TypeError("expected integer n, got {} with value {}".format(type(n), n))
     if n < 0:
         raise ValueError("expected n >= 0, got {}".format(n))
     k = k if k is not None else n  # not "k or n", since k = 0 is valid
+    if not isinstance(k, int):
+        raise TypeError("expected integer k, got {} with value {}".format(type(k), k))
     if k < 0:
         raise ValueError("expected k >= 0, got {}".format(k))
     out = []
@@ -350,11 +364,11 @@ def tail(iterable):
     """
     return drop(1, iterable)
 
-def first(iterable, default=None):
+def first(iterable, *, default=None):
     """Like nth, but return the first item."""
     return nth(0, iterable, default=default)
 
-def second(iterable, default=None):
+def second(iterable, *, default=None):
     """Like nth, but return the second item."""
     return nth(1, iterable, default=default)
 
@@ -363,6 +377,8 @@ def nth(n, iterable, *, default=None):
 
     The ``default`` is returned if there are fewer than ``n + 1`` items.
     """
+    if not isinstance(n, int):
+        raise TypeError("expected integer n, got {} with value {}".format(type(n), n))
     if n < 0:
         raise ValueError("expected n >= 0, got {}".format(n))
     it = drop(n - 1, iterable) if n else iter(iterable)
@@ -371,7 +387,7 @@ def nth(n, iterable, *, default=None):
     except StopIteration:
         return default
 
-def last(iterable, default=None):
+def last(iterable, *, default=None):
     """Return the last item from an iterable.
 
     We consume the iterable until it runs out of items, then return the
@@ -384,6 +400,17 @@ def last(iterable, default=None):
     d = deque(iterable, maxlen=1)  # C speed
     return d.pop() if d else default
 
+def scons(x, iterable):
+    """Prepend one element to the start of an iterable, return new iterable.
+
+    Same as ``itertools.chain((x,), iterable)``. The point is sometimes it is
+    convenient to be able to stuff one item in front of an existing iterator.
+    If ``iterable`` is a generator, this is somewhat like (stream-cons) in Racket.
+
+    If you need to prepend several values, just use ``itertools.chain``.
+    """
+    return chain((x,), iterable)
+
 def flatten(iterable, pred=None):
     """Recursively remove nested structure from iterable.
 
@@ -462,7 +489,7 @@ def iterate(f, *args):
     takes positional arguments; this will be unpacked to the argument list in
     the next call.
 
-    The yielded values are the tuples returned from the calls.
+    Or in other words, yield args, f(*args), f(*f(*args)), ...
     """
     while True:
         yield args
@@ -586,6 +613,9 @@ def test():
     assert tuple(zip_longest((1, 2, 3), (2, 4))) == ((1, 2), (2, 4), (3, None))  # itertools
     assert tuple(zipr_longest((1, 2, 3), (2, 4))) == ((3, 4), (2, 2), (1, None))
 
+    assert tuple(scons(0, range(1, 5))) == tuple(range(5))
+    assert tuple(tail(scons("foo", range(5)))) == tuple(range(5))
+
     reverse_one = curry(foldl, cons, nil)
     assert reverse_one(ll(1, 2, 3)) == ll(3, 2, 1)
 
@@ -750,7 +780,7 @@ def test():
         """Multiply an infinite stream by a constant."""
         return map(lambda x: c * x, s)
 
-    # will eventually crash (stack overflow, no TCO'd yield)
+    # will eventually crash (stack overflow)
     def ones_fp():
         yield 1
         yield from ones_fp()
@@ -762,27 +792,56 @@ def test():
         yield 1
         yield from adds(fibos_fp(), tail(fibos_fp()))
     def powers_of_2():
-        yield 2
+        yield 1
         yield from muls(powers_of_2(), 2)
     assert tuple(take(10, ones_fp())) == (1,) * 10
     assert tuple(take(10, nats_fp())) == tuple(range(10))
     assert tuple(take(10, fibos_fp())) == (1, 1, 2, 3, 5, 8, 13, 21, 34, 55)
-    assert tuple(take(10, powers_of_2())) == (2, 4, 8, 16, 32, 64, 128, 256, 512, 1024)
+    assert tuple(take(10, powers_of_2())) == (1, 2, 4, 8, 16, 32, 64, 128, 256, 512)
 
-    # not as FP as the above, but better Python
-    def ones_python():
-        while True:
-            yield 1
-    def nats_python(start=0):
-        return scanl(add, start, ones_python())
-    def fibos_python():
+    # The scanl equations are sometimes useful. The conditions
+    #   rs[0] = s0
+    #   rs[k+1] = rs[k] + xs[k]
+    # are equivalent with
+    #   rs = scanl(add, s0, xs)
+    # https://www.vex.net/~trebla/haskell/scanl.xhtml
+    def zs():  # s0 = 0, rs = [0, ...], xs = [0, ...]
+        yield from scanl(add, 0, zs())
+    def os():  # s0 = 1, rs = [1, ...], xs = [0, ...]
+        yield from scanl(add, 1, zs())
+    def ns(start=0):  # s0 = start, rs = [start, start+1, ...], xs = [1, ...]
+        yield from scanl(add, start, os())
+    def fs():  # s0 = 1, scons(1, rs) = fibos, xs = fibos
+        yield 1
+        yield from scanl(add, 1, fs())
+    def p2s():  # s0 = 1, rs = xs = [1, 2, 4, ...]
+        yield from scanl(add, 1, p2s())
+    assert tuple(take(10, zs())) == (0,) * 10
+    assert tuple(take(10, os())) == (1,) * 10
+    assert tuple(take(10, ns())) == tuple(range(10))
+    assert tuple(take(10, fs())) == (1, 1, 2, 3, 5, 8, 13, 21, 34, 55)
+    assert tuple(take(10, p2s())) == (1, 2, 4, 8, 16, 32, 64, 128, 256, 512)
+
+    # better Python: simple is better than complex
+    from itertools import repeat
+    def ones():
+        return repeat(1)
+    def nats(start=0):
+        return scanl(add, start, ones())
+    def fibos():
         a, b = 1, 1
         while True:
             yield a
             a, b = b, a + b
-    assert tuple(take(10, ones_python())) == (1,) * 10
-    assert tuple(take(10, nats_python())) == tuple(range(10))
-    assert tuple(take(10, fibos_python())) == (1, 1, 2, 3, 5, 8, 13, 21, 34, 55)
+    def pows():
+        x = 1
+        while True:
+            yield x
+            x *= 2
+    assert tuple(take(10, ones())) == (1,) * 10
+    assert tuple(take(10, nats())) == tuple(range(10))
+    assert tuple(take(10, fibos())) == (1, 1, 2, 3, 5, 8, 13, 21, 34, 55)
+    assert tuple(take(10, pows())) == (1, 2, 4, 8, 16, 32, 64, 128, 256, 512)
 
     # How to improve accuracy of numeric differentiation with FP tricks.
     #
